const nodepath = require('path');

require('dotenv').config({
  path: nodepath.join(__dirname, `./${process.env.NODE_ENV}.env`)
});

let utils=require('./utils');
let { assert, expect } = require('chai');
let { oauth2, client, setting } = require('../development');
let app = require('./app');
setting.debug(true);


let zoomBot=null;
let oauth2Client=null;
let userApp=null;
let oauthConnection=null;


describe('app action', function() {

  before(function() {
    
    oauth2Client = oauth2(
      process.env.clientId,
      process.env.clientSecret,
      process.env.redirect
    );
    oauthConnection=oauth2Client.connect();
    zoomBot = client(
      process.env.clientId,
      process.env.verificationToken,
      process.env.botJid
    )
    .configurate({errorHelp:true,help:true})
    .commands([
      {
        command:'settings',
        description: 'action a new meeting',
        hint:'<is_split_sidebar or default_sidebar_color> <value>'
      },
      {
        command:'fieldEdit',
        description:'field test',
        hint:'<some>'
      },
      {
        command: 'action8',
        description: 'action a new meeting',
        hint: ' <userName> <date>'
      },
      {
        command: 'reply',
        description: 'reply a new meeting',
        hint: ' <userName> <date>'
      }
    ])
    .defaultAuth(oauth2Client.connect());


    //message
    app.all('/webhook', async function(req, res) {
      let { body, headers } = req;
      // let data=zoomBot.parse({body,headers});
      try{
        let data=await zoomBot.handle({body,headers});
      }
      catch(e){
        console.log(e);
      }
      res.send('ok');
    });



    app.all('/oauth2', async function(req, res) {
      try {
        connection = await oauth2Client.connectByCode(req.query.code);
        //change userApp to current oauth2 user
        userApp = zoomBot.create({ auth: connection });
        res.send('ok');
      } catch (e) {
        res.send(e);
      }
    });



  });


  describe('event', async function() {
    it(`ready`, async function() {
      
      oauth2Client.on('tokens',async function(e){
        console.log(e);
      });
      zoomBot.on('actions',async function(e){
        let {info}=e;
        console.log(e);

      });

      zoomBot.on('commands', async function(e) {
        let { payload, data, type, command, message } = e; //origin message from IM is message
        let { toJid: to_jid, accountId: account_id, name } = payload;
        userApp=userApp||zoomBot.create({ auth: oauthConnection});
        try {//expired time is outtime,requres access_ by ref
          if(command==='reply'){
             await userApp.sendMessage({
               to_jid,
               account_id,
               body:{"type": "message", "text": `reply it `},
               header:{
                 text:'fox reply reply method'
               }
              });
          }
          else if(command==='fieldEdit'){
            await userApp.sendMessage({
              to_jid,
              account_id,
              body:{
                type: 'fields',
                items: [
                  {
                    "key":"ksdjf",
                    "value":"String",
                    "editable":true,
                    "style":{
                      //  "color":"String",
                      //  "bold":boolean,
                      //  "itatic":boolean
                    }
                 }
                ]
            },
              header:{
                text:'fox reply reply method'
              }
             });
          }
          else if(command==='settings'){
            await userApp.sendMessage({
              to_jid,
              account_id,
              // settings:{
              //   "is_split_sidebar":true,
              //   "default_sidebar_color":'#4D8486'
              // },
              body:[
              {"type": "message","sidebar_color":"#7E8626","text": `reply it `}
              // ,
              // { "type": "section",
              // "footer_icon": "https://platform.slack-edge.com/img/default_application_icon.png",
              // "footer": "footer test only for demo",
              // "ts": 123456789,
              // "sidebar_color": "#4D8486",
              // "sections": [{
              //     "type": "message",
              //     "text": "section 4 for test message editable",
              //     "editable": true
              // }
           
            ],
              header:{
                text:'reply method',
                style:{
                  itatic:true
                },
                "sub_head":{
                  "text":"some test",
                  "style":{
                      "color":"#868620",
                      "bold":false,
                      "itatic":true
                  }
              }
              }
             });
          }
          else if(command==='action'){
            let userInfo = await userApp.getUser();
            let channelList = await userApp.channelList(userInfo.id);
            if (channelList.length > 0) {
              let obj = channelList[1];
              let { group_jid } = obj;
              await userApp.sendMessage({
                to_jid: group_jid, 
                account_id: userInfo.account_id,
                body: { "type": "message", "text": `message from command ${command}` },
                header: { text: 'fox reply action method'}
              });
            }
          }
        }
        catch (e) {
          console.log(e);
        }
      });


    });
  });

  

  describe('wait', async function() {
    await utils.timeout(100000);
  });

  require('./listen.js');

});
